/* ------------------------------------------------------------- */
/* - STBee LED Flash on FreeRTOS sample                          */
/*                                  released by fumi2kick        */
/*                                  http://www.fumi2kick.com/    */
/*                                  rerofumi <rero2@yuumu.org> > */
/* ------------------------------------------------------------- */
/* update                                                        */
/*   Mar.13.2011 : first release                                 */
/* ------------------------------------------------------------- */
/* memo                                                          */
/*                                                               */
/* ------------------------------------------------------------- */

/* ----------------------------------- */
/* --- include                         */
/* ----------------------------------- */

#include "stm32f10x_conf.h"

#include "freertos/FreeRTOS.h"
#include "freertos/queue.h"
#include "freertos/semphr.h"
#include "freertos/task.h"


/* ----------------------------------- */
/* --- define                          */
/* ----------------------------------- */

#define	_BV(n)		( 1 << (n) )


/* ----------------------------------- */
/* --- local function prototyping      */
/* ----------------------------------- */

void vTask1(void *pcParam);
void vTask2(void *pcParam);


/* ----------------------------------- */
/* --- global scope                   */
/* ----------------------------------- */

/* --- global mutex */
xSemaphoreHandle  mutex_portb;


/* ------------------------------------------------------------- */
/* -----  main routine                                           */
/* ------------------------------------------------------------- */

int main(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
	
  /* --- enable to GPIO B */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
	
  /* --- gpio port initialize */
  GPIO_InitStructure.GPIO_Pin =  _BV(13) | _BV(14) | _BV(15);
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIOB, &GPIO_InitStructure);
  GPIO_ResetBits(GPIOB, _BV(13) | _BV(14) | _BV(15));

  /* --- create mutex */
  mutex_portb = xSemaphoreCreateMutex();

  /* --- create task */
  if (mutex_portb != NULL) {
    /* task spown */
    xTaskCreate( vTask1, (const signed char*)"Task1", 100, NULL, 1, NULL );
    xTaskCreate( vTask2, (const signed char*)"Task2", 100, NULL, 1, NULL );

    /* start scheduling, task running */
    vTaskStartScheduler();
  }

  /* ---end working */
  while(1) {
  }
}


/* ------------------------------------------------------------- */
/* -----  local function                                         */
/* ------------------------------------------------------------- */



/* ------------------------------------------------------------- */
/* -----  task                                                   */
/* ------------------------------------------------------------- */

/* ----------------------------- */
/* --- task1                     */
/* ----------------------------- */
void vTask1(void *pcParam)
{
  while(1) {
    xSemaphoreTake(mutex_portb, portMAX_DELAY);
    GPIO_ResetBits(GPIOB, _BV(13));
    xSemaphoreGive(mutex_portb);

    vTaskDelay(500 / portTICK_RATE_MS);

    xSemaphoreTake(mutex_portb, portMAX_DELAY);
    GPIO_SetBits(GPIOB, _BV(13));
    xSemaphoreGive(mutex_portb);

    vTaskDelay(500 / portTICK_RATE_MS);
  }
}


/* ----------------------------- */
/* --- task2                     */
/* ----------------------------- */
void vTask2(void *pcParam)
{
  while(1) {
    xSemaphoreTake(mutex_portb, portMAX_DELAY);
    GPIO_ResetBits(GPIOB, _BV(14));
    xSemaphoreGive(mutex_portb);

    vTaskDelay(700 / portTICK_RATE_MS);

    xSemaphoreTake(mutex_portb, portMAX_DELAY);
    GPIO_SetBits(GPIOB, _BV(14));
    xSemaphoreGive(mutex_portb);

    vTaskDelay(700 / portTICK_RATE_MS);
  }
}


